//
//  Copyright (c) 2006 by Autodesk, Inc.
//
//  By using this code, you are agreeing to the terms and conditions of
//  the License Agreement included in the documentation for this code.
//
//  AUTODESK MAKES NO WARRANTIES, EXPRESS OR IMPLIED, AS TO THE CORRECTNESS
//  OF THIS CODE OR ANY DERIVATIVE WORKS WHICH INCORPORATE IT. AUTODESK
//  PROVIDES THE CODE ON AN "AS-IS" BASIS AND EXPLICITLY DISCLAIMS ANY
//  LIABILITY, INCLUDING CONSEQUENTIAL AND INCIDENTAL DAMAGES FOR ERRORS,
//  OMISSIONS, AND OTHER PROBLEMS IN THE CODE.
//
//  Use, duplication, or disclosure by the U.S. Government is subject to
//  restrictions set forth in FAR 52.227-19 (Commercial Computer Software
//  Restricted Rights) and DFAR 252.227-7013(c)(1)(ii) (Rights in Technical
//  Data and Computer Software), as applicable.
//

//  $Header: //DWF/Development/Components/Internal/DWF Toolkit/v7.2.1/develop/global/src/dwf/presentation/utility/ContentPresentationContainer.cpp#2 $
//  $DateTime: 2006/05/13 05:37:53 $
//  $Author: gopalas $
//  $Change: 14579 $


#include "dwf/package/Constants.h"
#include "dwf/package/writer/DWFXMLSerializer.h"
#include "dwf/presentation/utility/ContentPresentationContainer.h"
#include "dwf/package/writer/PackageWriter.h"
using namespace DWFToolkit;

_DWFTK_API
DWFContentPresentationContainer::DWFContentPresentationContainer()
throw()
{
#ifndef DWFTK_READ_ONLY

    _oSerializer.is( this );

#endif
}

_DWFTK_API
DWFContentPresentationContainer::~DWFContentPresentationContainer()
throw()
{
    //
    // delete all presentations we own
    //
    DWFContentPresentation::tList::Iterator* piPresentations = getPresentations();

    if (piPresentations)
    {
        for (piPresentations->reset(); piPresentations->valid(); piPresentations->next())
        {
            DWFCORE_FREE_OBJECT(piPresentations->get());
        }
    }

    DWFCORE_FREE_OBJECT( piPresentations );
}

_DWFTK_API
void
DWFContentPresentationContainer::addPresentation( DWFContentPresentation* pPresentation )
throw( DWFException )
{
    if (pPresentation == NULL)
    {
        _DWFCORE_THROW( DWFInvalidArgumentException, L"A non-null presentation must be provided" );
    }

    size_t tIndex = _oPresentations.size();
    const DWFString& zID = pPresentation->id();
    //
    // if we have an ID check for duplicate and replace it if it already exists
    //
    if (zID.chars())
    {
        DWFContentPresentation::tList::Iterator* piPresentations = getPresentations();
        for( tIndex = 0; piPresentations && piPresentations->valid(); piPresentations->next(), tIndex++)
        {
            DWFContentPresentation* pExistingPresentation = piPresentations->get();
            if (pExistingPresentation->id() == zID)
            {
                _oPresentations.eraseAt( tIndex );
                DWFCORE_FREE_OBJECT( pExistingPresentation );
                break;
            }
        }

        DWFCORE_FREE_OBJECT( piPresentations );
    }

    //
    // set this container as the presentation's parent
    //
    pPresentation->setParent( this );
    _oPresentations.insertAt(pPresentation, tIndex);
}

_DWFTK_API
DWFContentPresentation::tList::Iterator*
DWFContentPresentationContainer::getPresentations( )
throw()
{
    return _oPresentations.iterator();
}

_DWFTK_API
void
DWFContentPresentationContainer::removePresentation( DWFContentPresentation* pPresentation,
                                                     bool                    bDelete )
throw( DWFException )
{
    _oPresentations.erase(pPresentation);
    
    if (bDelete)
    {
        DWFCORE_FREE_OBJECT(pPresentation);
    }
}

#ifndef DWFTK_READ_ONLY

void
DWFContentPresentationContainer::_Serializer::serializeXML( DWFXMLSerializer& rSerializer, unsigned int nFlags )
throw( DWFException )
{
    if (nFlags & DWFPackageWriter::eContentPresentation)
    {
        DWFContentPresentation::tList::Iterator* piContentPresentation = _pContainer->_oPresentations.iterator();

        if (piContentPresentation && piContentPresentation->valid())
        {
            DWFString zNamespace;

                //
                // namespace dictated by document and section type 
                //
            if (nFlags & DWFPackageWriter::eDescriptor)
            {
                zNamespace.assign( namespaceXML(nFlags) );
            }

            rSerializer.startElement( DWFXML::kzElement_Presentations, zNamespace );
            {
                if (piContentPresentation)
                {
                    for (; piContentPresentation->valid(); piContentPresentation->next())
                    {
                        piContentPresentation->get()->serializeXML( rSerializer, nFlags );
                    }
                }
            }

            rSerializer.endElement();
        }

        if (piContentPresentation)
        {
            DWFCORE_FREE_OBJECT( piContentPresentation );
        }
    }
    else
    {
        DWFContentPresentation::tList::Iterator* piContentPresentation = _pContainer->_oPresentations.iterator();

        if (piContentPresentation && piContentPresentation->valid())
        {
            for (; piContentPresentation->valid(); piContentPresentation->next())
            {
                piContentPresentation->get()->serializeXML( rSerializer, nFlags );
            }
        }

        if (piContentPresentation)
        {
            DWFCORE_FREE_OBJECT( piContentPresentation );
        }
    }
}

#endif

